@@ -10,6 +10,9 @@ from typedpy.structures import (
     ImmutableStructure,
     create_typed_field,
     NoneField,
+    FinalStructure,
+    ImmutableField,
+    unique,
 )
 from typedpy.fields import (
     Number,
@@ -34,7 +37,6 @@ from typedpy.fields import (
     Map,
     Tuple,
     StructureReference,
-    ImmutableField,
     Anything,
     SerializableField,
     Function,
@@ -75,3 +77,6 @@ from typedpy.extfields import (
     JSONString,
     EmailAddress,
 )
+
+from typedpy.errors import standard_readable_error_for_typedpy_exception
+
